@@ -15,6 +15,7 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"strings"
 
+	"github.com/Masterminds/semver"
 	"github.com/pkg/errors"
 	"gopkg.in/yaml.v2"
 
@@ -37,6 +38,7 @@ import (
 	"github.com/openshift/cluster-network-operator/pkg/names"
 	"github.com/openshift/cluster-network-operator/pkg/platform/openstack/util/cert"
 
+	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 
 	confv1 "github.com/openshift/api/config/v1"
@@ -58,6 +60,9 @@ const (
 	MinOctaviaVersionWithTagSupport        = "v2.5"
 	MinOctaviaVersionWithTimeouts          = "v2.1"
 	KuryrNamespace                         = "openshift-kuryr"
+	KuryrConfigMapName                     = "kuryr-config"
+	DNSNamespace                           = "openshift-dns"
+	DNSServiceName                         = "dns-default"
 	etcdPort                               = 2379
 	dnsPort                                = 53
 	apiPort                                = 6443
@@ -877,6 +882,18 @@ func ensureCertificate(kubeClient client.Client, caPEM []byte, privateKey []byte
 	}
 }
 
+func getConfigMap(kubeClient client.Client, namespace, name string) (*v1.ConfigMap, error) {
+	cm := &v1.ConfigMap{
+		TypeMeta:   metav1.TypeMeta{APIVersion: "v1", Kind: "ConfigMap"},
+		ObjectMeta: metav1.ObjectMeta{Name: name},
+	}
+	err := kubeClient.Get(context.TODO(), client.ObjectKey{Namespace: namespace, Name: name}, cm)
+	if err != nil {
+		return nil, err
+	}
+	return cm, nil
+}
+
 func getUserCACert(kubeClient client.Client) (string, error) {
 	cm := &v1.ConfigMap{
 		TypeMeta:   metav1.TypeMeta{APIVersion: "v1", Kind: "ConfigMap"},
@@ -889,6 +906,14 @@ func getUserCACert(kubeClient client.Client) (string, error) {
 	return string(cm.Data["ca-bundle.pem"]), nil
 }
 
+func getSavedAnnotation(kubeClient client.Client, annotation string) (string, error) {
+	cm, err := getConfigMap(kubeClient, KuryrNamespace, KuryrConfigMapName)
+	if err != nil {
+		return "", err
+	}
+	return cm.Annotations[annotation], nil
+}
+
 // Logs into OpenStack and creates all the resources that are required to run
 // Kuryr based on conf NetworkConfigSpec. Basically this includes service
 // network and subnet, pods subnetpool, security group and load balancer for
@@ -1235,6 +1260,39 @@ func BootstrapKuryr(conf *operv1.NetworkSpec, kubeClient client.Client) (*bootst
 		return nil, errors.Wrap(err, "failed to determine if Octavia supports double listeners")
 	}
 
+	octaviaVersion, err := getSavedAnnotation(kubeClient, names.KuryrOctaviaVersionAnnotation)
+	if err != nil && !apierrors.IsNotFound(err) {
+		return nil, errors.Wrap(err, "failed to get kuryr-config ConfigMap")
+	}
+
+	maxOctaviaVersion, err := getMaxOctaviaAPIVersion(lbClient)
+	if err != nil {
+		return nil, errors.Wrap(err, "failed to get max octavia api version")
+	}
+
+	// In case the Kuryr config-map is annotated with an Octavia version different
+	// than the current Octavia version, and older than the version that multiple
+	// listeners becomes available and the Octavia provider is amphora, an Octavia
+	// upgrade happened and UDP listeners are now allowed to be created.
+	// By recreating the OpenShift DNS service a new load balancer amphora is in
+	// place with all required listeners.
+	log.Print("Checking Octavia upgrade happened")
+	if octaviaVersion != "" {
+		savedOctaviaVersion := semver.MustParse(octaviaVersion)
+		multipleListenersVersion := semver.MustParse(MinOctaviaVersionWithMultipleListeners)
+		if !savedOctaviaVersion.Equal(maxOctaviaVersion) && savedOctaviaVersion.LessThan(multipleListenersVersion) && octaviaMultipleListenersSupport {
+			dnsService := &v1.Service{
+				TypeMeta:   metav1.TypeMeta{APIVersion: "v1", Kind: "Service"},
+				ObjectMeta: metav1.ObjectMeta{Name: DNSServiceName, Namespace: DNSNamespace},
+			}
+			err := kubeClient.Delete(context.TODO(), dnsService)
+			if err != nil {
+				return nil, errors.Wrapf(err, "Failed to delete %s Service", DNSServiceName)
+			}
+		}
+	}
+	octaviaVersion = maxOctaviaVersion.Original()
+
 	log.Print("Kuryr bootstrap finished")
 
 	res := bootstrap.BootstrapResult{
@@ -1248,6 +1306,7 @@ func BootstrapKuryr(conf *operv1.NetworkSpec, kubeClient client.Client) (*bootst
 			ClusterID:                clusterID,
 			OctaviaMultipleListeners: octaviaMultipleListenersSupport,
 			OpenStackCloud:           cloud,
+			OctaviaVersion:           octaviaVersion,
 			WebhookCA:                b64.StdEncoding.EncodeToString(ca),
 			WebhookCAKey:             b64.StdEncoding.EncodeToString(key),
 			WebhookKey:               b64.StdEncoding.EncodeToString(webhookKey),
